#!/bin/bash

# Copyright (c) 2007 Chuck Houpt. All rights reserved.

set -eux

# Setup reasonable defaults when running outside XCode.
PROJECT_DIR=${PROJECT_DIR:-$PWD}
BUILD_DIR=${PROJECT_DIR}/vbuild
#SDKROOT=/Developer/SDKs/MacOSX10.4u.sdk

# Set up reasonable default when running outside mutibuild.sh
#ARCH=${ARCH:-`arch`}
export MULTIBUILD_NCPU=${MULTIBUILD_NCPU:-`sysctl -n hw.ncpu`}

SOURCE_CACHE="$HOME/Library/Caches/net.habilis.source"

#SRC_DIR="$BUILD_DIR"/$ARCH/perlmods
SRC_DIR="$BUILD_DIR"/perlmods
PREFIX="$BUILD_DIR"/root
#EXEC_PREFIX="$PREFIX"/$ARCH
EXEC_PREFIX="$PREFIX"
#eval SDKROOT=\$SDKROOT_$ARCH

#export CFLAGS="-arch $ARCH -Os -isysroot $SDKROOT"
#export LDFLAGS="-arch $ARCH -Wl,-syslibroot,$SDKROOT"
#export CC="gcc -arch ppc -arch i386"
#export CFLAGS="-arch ppc -arch i386 -Os -isysroot $SDKROOT"
#export LDFLAGS="-arch ppc -arch i386 -Wl,-syslibroot,$SDKROOT"

# Add installed binaries to PATH
PATH="$EXEC_PREFIX"/bin:"$EXEC_PREFIX"/usr/bin:$PATH
export DYLD_LIBRARY_PATH="$EXEC_PREFIX"/lib

# This is what local::lib sets:
#export PERL_LOCAL_LIB_ROOT="/Users/chuck/perl5";
#export PERL_MB_OPT="--install_base /Users/chuck/perl5";
#export PERL_MM_OPT="INSTALL_BASE=/Users/chuck/perl5";
#export PERL5LIB="/Users/chuck/perl5/lib/perl5/darwin-thread-multi-2level:/Users/chuck/perl5/lib/perl5";
#export PATH="/Users/chuck/perl5/bin:$PATH";

export VERSIONER_PERL_VERSION=5.18
export PERL_LOCAL_LIB_ROOT="$PREFIX";
export PERL_MB_OPT="--install_base $PREFIX";
export PERL_MM_OPT="INSTALL_BASE=$PREFIX";
export PERL5LIB="$PREFIX/lib/perl5/darwin-thread-multi-2level:$PREFIX/lib/perl5";
export PATH="$PREFIX/bin:$PATH";


function buildmodule ()
{
    local module_url="$1"
	local module=`basename $module_url`

	if [ ! -r "$SOURCE_CACHE"/${module}.tar.gz ]
	then
		mkdir -p "$SOURCE_CACHE"
		pushd "$SOURCE_CACHE"
		curl -fsO http://www.cpan.org/modules/by-module/${module_url}.tar.gz
		popd
	fi
    
	if [ ! -d ${module} ]
	then
#	rm -rf ${module}
    tar xzf "$SOURCE_CACHE"/${module}.tar.gz
	fi
	
    pushd ${module}

	if [ ! -f Makefile ]
	then
		# Adding CC and LDDL flags for -I/-L $PREFIX paths overides the default flags,
		# so they have to be added in this kludgy way...
		perl Makefile.PL \
		CCFLAGS="-arch x86_64 -arch i386 -g -pipe -fno-common -DPERL_DARWIN -fno-strict-aliasing -fstack-protector -I$PREFIX/include" \
		LDDLFLAGS="-arch x86_64 -arch i386 -bundle -undefined dynamic_lookup -fstack-protector -L$PREFIX/lib -losp"
	fi

	make test install

    popd
}

# Not strictly correct, just checking for any build result.
if [ -f "$PREFIX"/lib/perl5.18/darwin-thread-multi-2level/auto/SGML/Parser/OpenSP/OpenSP.bundle ] ; then exit ; fi

mkdir -p "$SRC_DIR"

#export PERLLIB="$PREFIX"/lib:"$PREFIX"/lib/darwin-thread-multi-2level
#export PERL5LIB="$PREFIX"/lib:"$EXEC_PREFIX"/lib
# Force default answers to MakeMaker prompts
#export PERL_MM_USE_DEFAULT=1

pushd "$SRC_DIR"

# needed for SSI, if only it would work:  Date/Date-Format? HTML/HTML-SimpleParse-0.12 CGI/CGI-SSI-0.91 CGI/CGI-SSI_Parser-0.01

# For 10.5, removed Test/Test-Builder-Tester-1.01 because 10.5 has a newer version
#for modurl in XML/XML-NamespaceSupport-1.09  XML/XML-LibXML-Common-0.13 XML/XML-SAX-0.16   XML/XML-LibXML-1.63  Encode/Encode-HanExtra-0.10 Encode/Encode-JIS2K-0.02 URI/URI-1.35 Net/Net-IP-1.24 Text/Text-Iconv-1.4 Config/Config-General-2.31 Set/Set-IntSpan-1.13 HTML/HTML-Tagset-3.10 HTML/libwww-perl-5.805 HTML/HTML-Template-2.8 HTML/HTML-Parser-3.56 HTML/HTML-Encoding-0.53 Class/Class-Accessor-0.30 Sub/Sub-Uplevel-0.13  Test/Test-Exception-0.25 SGML/SGML-Parser-OpenSP-0.994
# for 5.12, removed URI
# for 5.18, removed XML/XML-LibXML-2.0007 CGI/CGI.pm-3.51
for modurl in  SGML/SGML-Parser-OpenSP-0.994 Config/Config-General-2.56 Encode/Encode-HanExtra-0.23 Encode/Encode-JIS2K-0.02 HTML/HTML-Parser-3.71 HTML/HTML-Encoding-0.61 HTML/HTML-Template-2.9 JSON/JSON-2.90
do
  buildmodule ${modurl}
done

popd

# Rename
mv "$PREFIX"/lib/perl5 "$PREFIX"/lib/perl5.18

# Add perl5.10.0 libraries for 10.6
rsync -a "$PROJECT_DIR"/perl5.10.0 "$PREFIX"/lib
